@@ -30,12 +30,17 @@
  *      Filters
  *           PIX      *pixSelectBySize()
  *           PIXA     *pixaSelectBySize()
- *           PIX      *pixSelectByAreaPerimRatio()
- *           PIXA     *pixaSelectByAreaPerimRatio()
+ *           NUMA     *pixaMakeSizeIndicator()
+ *
+ *           PIX      *pixSelectByPerimToAreaRatio()
+ *           PIXA     *pixaSelectByPerimToAreaRatio()
+ *           PIX      *pixSelectByPerimSizeRatio()
+ *           PIXA     *pixaSelectByPerimSizeRatio()
  *           PIX      *pixSelectByAreaFraction()
  *           PIXA     *pixaSelectByAreaFraction()
  *           PIX      *pixSelectByWidthHeightRatio()
  *           PIXA     *pixaSelectByWidthHeightRatio()
+ *
  *           PIXA     *pixaSelectWithIndicator()
  *           l_int32   pixRemoveWithIndicator()
  *           l_int32   pixAddWithIndicator()
@@ -47,13 +52,24 @@
  *           PIXA     *pixaSortByIndex()
  *           PIXAA    *pixaSort2dByIndex()
  *
+ *      Pixa and Pixaa range selection
+ *           PIXA     *pixaSelectRange()
+ *           PIXAA    *pixaaSelectRange()
+ *
+ *      Pixa and Pixaa scaling
+ *           PIXAA    *pixaaScaleToSize()
+ *           PIXAA    *pixaaScaleToSizeVar()
+ *           PIXA     *pixaScaleToSize()
+ *
  *      Miscellaneous
  *           PIXA     *pixaAddBorderGeneral()
  *           PIXA     *pixaaFlattenToPixa()
+ *           l_int32   pixaaSizeRange()
  *           l_int32   pixaSizeRange()
  *           PIXA     *pixaClipToPix()
  *           l_int32   pixaAnyColormaps()
  *           l_int32   pixaGetDepthInfo()
+ *           PIXA     *pixaConvertToSameDepth()
  *           l_int32   pixaEqual()
  */
 
@@ -63,7 +79,7 @@
     /* For more than this number of c.c. in a binarized image of
      * semi-perimeter (w + h) about 5000 or less, the O(n) binsort
      * is faster than the O(nlogn) shellsort.  */
-static const l_int32   MIN_COMPS_FOR_BIN_SORT = 500;
+static const l_int32   MIN_COMPS_FOR_BIN_SORT = 200;
 
 
 /*---------------------------------------------------------------------*
@@ -93,14 +109,16 @@ static const l_int32   MIN_COMPS_FOR_BIN_SORT = 500;
  * This is the easiest way to do simple filtering.  These functions
  * are named pixSelectBy*() and pixaSelectBy*(), where the '*' is one of:
  *        Size
- *        AreaPerimRatio
+ *        PerimToAreaRatio
+ *        PerimSizeRatio
  *        AreaFraction
  *        WidthHeightRatio
  *
  * For more complicated filtering, use the general method (2).
  * The numa indicator functions for a pixa are generated by these functions:
  *        pixaFindDimensions()
- *        pixaFindAreaPerimRatio()
+ *        pixaFindPerimToAreaRatio()
+ *        pixaFindPerimSizeRatio()
  *        pixaFindAreaFraction()
  *        pixaCountPixels()
  *        pixaFindWidthHeightRatio()
@@ -109,7 +127,7 @@ static const l_int32   MIN_COMPS_FOR_BIN_SORT = 500;
  * Here is an illustration using the general method.  Suppose you want
  * all 8-connected components that have a height greater than 40 pixels,
  * a width not more than 30 pixels, between 150 and 300 fg pixels,
- * and an area-to-perimeter ratio between 2.5 and 4.0.
+ * and a perimeter-to-size ratio between 1.2 and 2.0.
  *
  *        // Generate the pixa of 8 cc pieces.
  *    boxa = pixConnComp(pixs, &pixa, 8);
@@ -117,7 +135,7 @@ static const l_int32   MIN_COMPS_FOR_BIN_SORT = 500;
  *        // Extract the data we need about each component.
  *    pixaFindDimensions(pixa, &naw, &nah);
  *    nas = pixaCountPixels(pixa);
- *    nar = pixaFindAreaPerimRatio(pixa);
+ *    nar = pixaFindPerimSizeRatio(pixa);
  *
  *        // Build the indicator arrays for the set of components,
  *        // based on thresholds and selection criteria.
@@ -125,8 +143,8 @@ static const l_int32   MIN_COMPS_FOR_BIN_SORT = 500;
  *    na2 = numaMakeThresholdIndicator(naw, 30, L_SELECT_IF_LTE);
  *    na3 = numaMakeThresholdIndicator(nas, 150, L_SELECT_IF_GTE);
  *    na4 = numaMakeThresholdIndicator(nas, 300, L_SELECT_IF_LTE);
- *    na5 = numaMakeThresholdIndicator(nar, 2.5, L_SELECT_IF_GTE);
- *    na6 = numaMakeThresholdIndicator(nar, 4.0, L_SELECT_IF_LGE);
+ *    na5 = numaMakeThresholdIndicator(nar, 1.2, L_SELECT_IF_GTE);
+ *    na6 = numaMakeThresholdIndicator(nar, 2.0, L_SELECT_IF_LTE);
  *
  *        // Combine the indicator arrays logically to find
  *        // the components that will be retained.
@@ -208,27 +226,26 @@ PIXA    *pixas, *pixad;
     boxaDestroy(&boxa);
     pixaDestroy(&pixas);
 
-        /* Render the result */
     if (!changed) {
         pixaDestroy(&pixad);
         return pixCopy(NULL, pixs);
     }
-    else {
-        if (pchanged) *pchanged = TRUE;
-        pixGetDimensions(pixs, &w, &h, NULL);
-        count = pixaGetCount(pixad);
-        if (count == 0)  /* return empty pix */
-            pixd = pixCreateTemplate(pixs);
-        else {
-            pixd = pixaDisplay(pixad, w, h);
-            pixCopyResolution(pixd, pixs);
-            pixCopyColormap(pixd, pixs);
-            pixCopyText(pixd, pixs);
-            pixCopyInputFormat(pixd, pixs);
-        }
-        pixaDestroy(&pixad);
-        return pixd;
+
+        /* Render the result */
+    if (pchanged) *pchanged = TRUE;
+    pixGetDimensions(pixs, &w, &h, NULL);
+    count = pixaGetCount(pixad);
+    if (count == 0) {  /* return empty pix */
+        pixd = pixCreateTemplate(pixs);
+    } else {
+        pixd = pixaDisplay(pixad, w, h);
+        pixCopyResolution(pixd, pixs);
+        pixCopyColormap(pixd, pixs);
+        pixCopyText(pixd, pixs);
+        pixCopyInputFormat(pixd, pixs);
     }
+    pixaDestroy(&pixad);
+    return pixd;
 }
 
 
@@ -263,9 +280,8 @@ pixaSelectBySize(PIXA     *pixas,
                  l_int32   relation,
                  l_int32  *pchanged)
 {
-BOXA    *boxa;
-NUMA    *na;
-PIXA    *pixad;
+NUMA  *na;
+PIXA  *pixad;
 
     PROCNAME("pixaSelectBySize");
 
@@ -279,9 +295,7 @@ PIXA    *pixad;
         return (PIXA *)ERROR_PTR("invalid relation", procName, NULL);
 
         /* Compute the indicator array for saving components */
-    boxa = pixaGetBoxa(pixas, L_CLONE);
-    na = boxaMakeSizeIndicator(boxa, width, height, type, relation);
-    boxaDestroy(&boxa);
+    na = pixaMakeSizeIndicator(pixas, width, height, type, relation);
 
         /* Filter to get output */
     pixad = pixaSelectWithIndicator(pixas, na, pchanged);
@@ -292,10 +306,98 @@ PIXA    *pixad;
 
 
 /*!
- *  pixSelectByAreaPerimRatio()
+ *  pixaMakeSizeIndicator()
+ *
+ *      Input:  pixa
+ *              width, height (threshold dimensions)
+ *              type (L_SELECT_WIDTH, L_SELECT_HEIGHT,
+ *                    L_SELECT_IF_EITHER, L_SELECT_IF_BOTH)
+ *              relation (L_SELECT_IF_LT, L_SELECT_IF_GT,
+ *                        L_SELECT_IF_LTE, L_SELECT_IF_GTE)
+ *      Return: na (indicator array), or null on error
+ *
+ *  Notes:
+ *      (1) The args specify constraints on the size of the
+ *          components that are kept.
+ *      (2) If the selection type is L_SELECT_WIDTH, the input
+ *          height is ignored, and v.v.
+ *      (3) To keep small components, use relation = L_SELECT_IF_LT or
+ *          L_SELECT_IF_LTE.
+ *          To keep large components, use relation = L_SELECT_IF_GT or
+ *          L_SELECT_IF_GTE.
+ */
+NUMA *
+pixaMakeSizeIndicator(PIXA     *pixa,
+                      l_int32   width,
+                      l_int32   height,
+                      l_int32   type,
+                      l_int32   relation)
+{
+l_int32  i, n, w, h, ival;
+NUMA    *na;
+
+    PROCNAME("pixaMakeSizeIndicator");
+
+    if (!pixa)
+        return (NUMA *)ERROR_PTR("pixa not defined", procName, NULL);
+    if (type != L_SELECT_WIDTH && type != L_SELECT_HEIGHT &&
+        type != L_SELECT_IF_EITHER && type != L_SELECT_IF_BOTH)
+        return (NUMA *)ERROR_PTR("invalid type", procName, NULL);
+    if (relation != L_SELECT_IF_LT && relation != L_SELECT_IF_GT &&
+        relation != L_SELECT_IF_LTE && relation != L_SELECT_IF_GTE)
+        return (NUMA *)ERROR_PTR("invalid relation", procName, NULL);
+
+    n = pixaGetCount(pixa);
+    na = numaCreate(n);
+    for (i = 0; i < n; i++) {
+        ival = 0;
+        pixaGetPixDimensions(pixa, i, &w, &h, NULL);
+        switch (type)
+        {
+        case L_SELECT_WIDTH:
+            if ((relation == L_SELECT_IF_LT && w < width) ||
+                (relation == L_SELECT_IF_GT && w > width) ||
+                (relation == L_SELECT_IF_LTE && w <= width) ||
+                (relation == L_SELECT_IF_GTE && w >= width))
+                ival = 1;
+            break;
+        case L_SELECT_HEIGHT:
+            if ((relation == L_SELECT_IF_LT && h < height) ||
+                (relation == L_SELECT_IF_GT && h > height) ||
+                (relation == L_SELECT_IF_LTE && h <= height) ||
+                (relation == L_SELECT_IF_GTE && h >= height))
+                ival = 1;
+            break;
+        case L_SELECT_IF_EITHER:
+            if (((relation == L_SELECT_IF_LT) && (w < width || h < height)) ||
+                ((relation == L_SELECT_IF_GT) && (w > width || h > height)) ||
+               ((relation == L_SELECT_IF_LTE) && (w <= width || h <= height)) ||
+                ((relation == L_SELECT_IF_GTE) && (w >= width || h >= height)))
+                    ival = 1;
+            break;
+        case L_SELECT_IF_BOTH:
+            if (((relation == L_SELECT_IF_LT) && (w < width && h < height)) ||
+                ((relation == L_SELECT_IF_GT) && (w > width && h > height)) ||
+               ((relation == L_SELECT_IF_LTE) && (w <= width && h <= height)) ||
+                ((relation == L_SELECT_IF_GTE) && (w >= width && h >= height)))
+                    ival = 1;
+            break;
+        default:
+            L_WARNING("can't get here!\n", procName);
+            break;
+        }
+        numaAddNumber(na, ival);
+    }
+
+    return na;
+}
+
+
+/*!
+ *  pixSelectByPerimToAreaRatio()
  *
  *      Input:  pixs (1 bpp)
- *              thresh (threshold ratio of interior/boundary pixels)
+ *              thresh (threshold ratio of fg boundary to fg pixels)
  *              connectivity (4 or 8)
  *              type (L_SELECT_IF_LT, L_SELECT_IF_GT,
  *                    L_SELECT_IF_LTE, L_SELECT_IF_GTE)
@@ -307,25 +409,25 @@ PIXA    *pixad;
  *          components that are kept.
  *      (2) If unchanged, returns a copy of pixs.  Otherwise,
  *          returns a new pix with the filtered components.
- *      (3) This filters "thin" components, where a thin component
- *          is defined to have a ratio of interior to boundary pixels
+ *      (3) This filters "thick" components, where a thick component
+ *          is defined to have a ratio of boundary to interior pixels
  *          that is smaller than a given threshold value.
- *      (4) Use L_SELECT_IF_LT or L_SELECT_IF_LTE to save the thin
+ *      (4) Use L_SELECT_IF_LT or L_SELECT_IF_LTE to save the thicker
  *          components, and L_SELECT_IF_GT or L_SELECT_IF_GTE to remove them.
  */
 PIX *
-pixSelectByAreaPerimRatio(PIX       *pixs,
-                         l_float32  thresh,
-                         l_int32    connectivity,
-                         l_int32    type,
-                         l_int32   *pchanged)
+pixSelectByPerimToAreaRatio(PIX       *pixs,
+                            l_float32  thresh,
+                            l_int32    connectivity,
+                            l_int32    type,
+                            l_int32   *pchanged)
 {
 l_int32  w, h, empty, changed, count;
 BOXA    *boxa;
 PIX     *pixd;
 PIXA    *pixas, *pixad;
 
-    PROCNAME("pixSelectByAreaPerimRatio");
+    PROCNAME("pixSelectByPerimToAreaRatio");
 
     if (!pixs)
         return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
@@ -343,39 +445,38 @@ PIXA    *pixas, *pixad;
 
         /* Filter thin components */
     boxa = pixConnComp(pixs, &pixas, connectivity);
-    pixad = pixaSelectByAreaPerimRatio(pixas, thresh, type, &changed);
+    pixad = pixaSelectByPerimToAreaRatio(pixas, thresh, type, &changed);
     boxaDestroy(&boxa);
     pixaDestroy(&pixas);
 
-        /* Render the result */
     if (!changed) {
         pixaDestroy(&pixad);
         return pixCopy(NULL, pixs);
     }
-    else {
-        if (pchanged) *pchanged = TRUE;
-        pixGetDimensions(pixs, &w, &h, NULL);
-        count = pixaGetCount(pixad);
-        if (count == 0)  /* return empty pix */
-            pixd = pixCreateTemplate(pixs);
-        else {
-            pixd = pixaDisplay(pixad, w, h);
-            pixCopyResolution(pixd, pixs);
-            pixCopyColormap(pixd, pixs);
-            pixCopyText(pixd, pixs);
-            pixCopyInputFormat(pixd, pixs);
-        }
-        pixaDestroy(&pixad);
-        return pixd;
+
+        /* Render the result */
+    if (pchanged) *pchanged = TRUE;
+    pixGetDimensions(pixs, &w, &h, NULL);
+    count = pixaGetCount(pixad);
+    if (count == 0) {  /* return empty pix */
+        pixd = pixCreateTemplate(pixs);
+    } else {
+        pixd = pixaDisplay(pixad, w, h);
+        pixCopyResolution(pixd, pixs);
+        pixCopyColormap(pixd, pixs);
+        pixCopyText(pixd, pixs);
+        pixCopyInputFormat(pixd, pixs);
     }
+    pixaDestroy(&pixad);
+    return pixd;
 }
 
 
 /*!
- *  pixaSelectByAreaPerimRatio()
+ *  pixaSelectByPerimToAreaRatio()
  *
  *      Input:  pixas
- *              thresh (threshold ratio of interior/boundary pixels)
+ *              thresh (threshold ratio of fg boundary to fg pixels)
  *              type (L_SELECT_IF_LT, L_SELECT_IF_GT,
  *                    L_SELECT_IF_LTE, L_SELECT_IF_GTE)
  *              &changed (<optional return> 1 if changed; 0 if clone returned)
@@ -384,22 +485,18 @@ PIXA    *pixas, *pixad;
  *  Notes:
  *      (1) Returns a pixa clone if no components are removed.
  *      (2) Uses pix and box clones in the new pixa.
- *      (3) We define a thin component here to be one with a ratio
- *          of interior to boundary pixels that is smaller than a given
- *          threshold value.
- *      (4) Use L_SELECT_IF_LT or L_SELECT_IF_LTE to save the thin
- *          components, and L_SELECT_IF_GT or L_SELECT_IF_GTE to remove them.
+ *      (3) See pixSelectByPerimToAreaRatio().
  */
 PIXA *
-pixaSelectByAreaPerimRatio(PIXA      *pixas,
-                           l_float32  thresh,
-                           l_int32    type,
-                           l_int32   *pchanged)
+pixaSelectByPerimToAreaRatio(PIXA      *pixas,
+                             l_float32  thresh,
+                             l_int32    type,
+                             l_int32   *pchanged)
 {
 NUMA  *na, *nai;
 PIXA  *pixad;
 
-    PROCNAME("pixaSelectByAreaPerimRatio");
+    PROCNAME("pixaSelectByPerimToAreaRatio");
 
     if (!pixas)
         return (PIXA *)ERROR_PTR("pixas not defined", procName, NULL);
@@ -408,7 +505,134 @@ PIXA  *pixad;
         return (PIXA *)ERROR_PTR("invalid type", procName, NULL);
 
         /* Compute component ratios. */
-    na = pixaFindAreaPerimRatio(pixas);
+    na = pixaFindPerimToAreaRatio(pixas);
+
+        /* Generate indicator array for elements to be saved. */
+    nai = numaMakeThresholdIndicator(na, thresh, type);
+    numaDestroy(&na);
+
+        /* Filter to get output */
+    pixad = pixaSelectWithIndicator(pixas, nai, pchanged);
+
+    numaDestroy(&nai);
+    return pixad;
+}
+
+
+/*!
+ *  pixSelectByPerimSizeRatio()
+ *
+ *      Input:  pixs (1 bpp)
+ *              thresh (threshold ratio of fg boundary to fg pixels)
+ *              connectivity (4 or 8)
+ *              type (L_SELECT_IF_LT, L_SELECT_IF_GT,
+ *                    L_SELECT_IF_LTE, L_SELECT_IF_GTE)
+ *              &changed (<optional return> 1 if changed; 0 if clone returned)
+ *      Return: pixd, or null on error
+ *
+ *  Notes:
+ *      (1) The args specify constraints on the size of the
+ *          components that are kept.
+ *      (2) If unchanged, returns a copy of pixs.  Otherwise,
+ *          returns a new pix with the filtered components.
+ *      (3) This filters components with smooth vs. dendritic shape, using
+ *          the ratio of the fg boundary pixels to the circumference of
+ *          the bounding box, and comparing it to a threshold value.
+ *      (4) Use L_SELECT_IF_LT or L_SELECT_IF_LTE to save the smooth
+ *          boundary components, and L_SELECT_IF_GT or L_SELECT_IF_GTE
+ *          to remove them.
+ */
+PIX *
+pixSelectByPerimSizeRatio(PIX       *pixs,
+                          l_float32  thresh,
+                          l_int32    connectivity,
+                          l_int32    type,
+                          l_int32   *pchanged)
+{
+l_int32  w, h, empty, changed, count;
+BOXA    *boxa;
+PIX     *pixd;
+PIXA    *pixas, *pixad;
+
+    PROCNAME("pixSelectByPerimSizeRatio");
+
+    if (!pixs)
+        return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
+    if (connectivity != 4 && connectivity != 8)
+        return (PIX *)ERROR_PTR("connectivity not 4 or 8", procName, NULL);
+    if (type != L_SELECT_IF_LT && type != L_SELECT_IF_GT &&
+        type != L_SELECT_IF_LTE && type != L_SELECT_IF_GTE)
+        return (PIX *)ERROR_PTR("invalid type", procName, NULL);
+    if (pchanged) *pchanged = FALSE;
+
+        /* Check if any components exist */
+    pixZero(pixs, &empty);
+    if (empty)
+        return pixCopy(NULL, pixs);
+
+        /* Filter thin components */
+    boxa = pixConnComp(pixs, &pixas, connectivity);
+    pixad = pixaSelectByPerimSizeRatio(pixas, thresh, type, &changed);
+    boxaDestroy(&boxa);
+    pixaDestroy(&pixas);
+
+    if (!changed) {
+        pixaDestroy(&pixad);
+        return pixCopy(NULL, pixs);
+    }
+
+        /* Render the result */
+    if (pchanged) *pchanged = TRUE;
+    pixGetDimensions(pixs, &w, &h, NULL);
+    count = pixaGetCount(pixad);
+    if (count == 0) {  /* return empty pix */
+        pixd = pixCreateTemplate(pixs);
+    } else {
+        pixd = pixaDisplay(pixad, w, h);
+        pixCopyResolution(pixd, pixs);
+        pixCopyColormap(pixd, pixs);
+        pixCopyText(pixd, pixs);
+        pixCopyInputFormat(pixd, pixs);
+    }
+    pixaDestroy(&pixad);
+    return pixd;
+}
+
+
+/*!
+ *  pixaSelectByPerimSizeRatio()
+ *
+ *      Input:  pixas
+ *              thresh (threshold ratio of fg boundary to b.b. circumference)
+ *              type (L_SELECT_IF_LT, L_SELECT_IF_GT,
+ *                    L_SELECT_IF_LTE, L_SELECT_IF_GTE)
+ *              &changed (<optional return> 1 if changed; 0 if clone returned)
+ *      Return: pixad, or null on error
+ *
+ *  Notes:
+ *      (1) Returns a pixa clone if no components are removed.
+ *      (2) Uses pix and box clones in the new pixa.
+ *      (3) See pixSelectByPerimSizeRatio().
+ */
+PIXA *
+pixaSelectByPerimSizeRatio(PIXA      *pixas,
+                           l_float32  thresh,
+                           l_int32    type,
+                           l_int32   *pchanged)
+{
+NUMA  *na, *nai;
+PIXA  *pixad;
+
+    PROCNAME("pixaSelectByPerimSizeRatio");
+
+    if (!pixas)
+        return (PIXA *)ERROR_PTR("pixas not defined", procName, NULL);
+    if (type != L_SELECT_IF_LT && type != L_SELECT_IF_GT &&
+        type != L_SELECT_IF_LTE && type != L_SELECT_IF_GTE)
+        return (PIXA *)ERROR_PTR("invalid type", procName, NULL);
+
+        /* Compute component ratios. */
+    na = pixaFindPerimSizeRatio(pixas);
 
         /* Generate indicator array for elements to be saved. */
     nai = numaMakeThresholdIndicator(na, thresh, type);
@@ -478,27 +702,26 @@ PIXA    *pixas, *pixad;
     boxaDestroy(&boxa);
     pixaDestroy(&pixas);
 
-        /* Render the result */
     if (!changed) {
         pixaDestroy(&pixad);
         return pixCopy(NULL, pixs);
     }
-    else {
-        if (pchanged) *pchanged = TRUE;
-        pixGetDimensions(pixs, &w, &h, NULL);
-        count = pixaGetCount(pixad);
-        if (count == 0)  /* return empty pix */
-            pixd = pixCreateTemplate(pixs);
-        else {
-            pixd = pixaDisplay(pixad, w, h);
-            pixCopyResolution(pixd, pixs);
-            pixCopyColormap(pixd, pixs);
-            pixCopyText(pixd, pixs);
-            pixCopyInputFormat(pixd, pixs);
-        }
-        pixaDestroy(&pixad);
-        return pixd;
+
+        /* Render the result */
+    if (pchanged) *pchanged = TRUE;
+    pixGetDimensions(pixs, &w, &h, NULL);
+    count = pixaGetCount(pixad);
+    if (count == 0) {  /* return empty pix */
+        pixd = pixCreateTemplate(pixs);
+    } else {
+        pixd = pixaDisplay(pixad, w, h);
+        pixCopyResolution(pixd, pixs);
+        pixCopyColormap(pixd, pixs);
+        pixCopyText(pixd, pixs);
+        pixCopyInputFormat(pixd, pixs);
     }
+    pixaDestroy(&pixad);
+    return pixd;
 }
 
 
@@ -608,27 +831,26 @@ PIXA    *pixas, *pixad;
     boxaDestroy(&boxa);
     pixaDestroy(&pixas);
 
-        /* Render the result */
     if (!changed) {
         pixaDestroy(&pixad);
         return pixCopy(NULL, pixs);
     }
-    else {
-        if (pchanged) *pchanged = TRUE;
-        pixGetDimensions(pixs, &w, &h, NULL);
-        count = pixaGetCount(pixad);
-        if (count == 0)  /* return empty pix */
-            pixd = pixCreateTemplate(pixs);
-        else {
-            pixd = pixaDisplay(pixad, w, h);
-            pixCopyResolution(pixd, pixs);
-            pixCopyColormap(pixd, pixs);
-            pixCopyText(pixd, pixs);
-            pixCopyInputFormat(pixd, pixs);
-        }
-        pixaDestroy(&pixad);
-        return pixd;
+
+        /* Render the result */
+    if (pchanged) *pchanged = TRUE;
+    pixGetDimensions(pixs, &w, &h, NULL);
+    count = pixaGetCount(pixad);
+    if (count == 0) {  /* return empty pix */
+        pixd = pixCreateTemplate(pixs);
+    } else {
+        pixd = pixaDisplay(pixad, w, h);
+        pixCopyResolution(pixd, pixs);
+        pixCopyColormap(pixd, pixs);
+        pixCopyText(pixd, pixs);
+        pixCopyInputFormat(pixd, pixs);
     }
+    pixaDestroy(&pixad);
+    return pixd;
 }
 
 
@@ -695,13 +917,15 @@ PIXA  *pixad;
  *      (1) Returns a pixa clone if no components are removed.
  *      (2) Uses pix and box clones in the new pixa.
  *      (3) The indicator numa has values 0 (ignore) and 1 (accept).
+ *      (4) If the source boxa is not fully populated, it is left
+ *          empty in the dest pixa.
  */
 PIXA *
 pixaSelectWithIndicator(PIXA     *pixas,
                         NUMA     *na,
                         l_int32  *pchanged)
 {
-l_int32  i, n, ival, nsave;
+l_int32  i, n, nbox, ival, nsave;
 BOX     *box;
 PIX     *pixt;
 PIXA    *pixad;
@@ -726,13 +950,16 @@ PIXA    *pixad;
     }
     if (pchanged) *pchanged = TRUE;
     pixad = pixaCreate(nsave);
+    nbox = pixaGetBoxaCount(pixas);
     for (i = 0; i < n; i++) {
         numaGetIValue(na, i, &ival);
         if (ival == 0) continue;
         pixt = pixaGetPix(pixas, i, L_CLONE);
-        box = pixaGetBox(pixas, i, L_CLONE);
         pixaAddPix(pixad, pixt, L_INSERT);
-        pixaAddBox(pixad, box, L_INSERT);
+        if (nbox == n) {   /* fully populated boxa */
+            box = pixaGetBox(pixas, i, L_CLONE);
+            pixaAddBox(pixad, box, L_INSERT);
+        }
     }
 
     return pixad;
@@ -842,14 +1069,16 @@ PIX     *pix;
 /*!
  *  pixaRenderComponent()
  *
- *      Input:  pixs (1 bpp pix; can be null)
- *              pixa (of connected components, one of which will be put
- *                    into pixs)
+ *      Input:  pixs (<optional> 1 bpp pix)
+ *              pixa (of 1 bpp connected components, one of which will
+ *                    be rendered in pixs, with its origin determined
+ *                    by the associated box.)
  *              index (of component to be rendered)
  *      Return: pixd, or null on error
  *
  *  Notes:
- *      (1) If pixs is null, this generates an empty pix.
+ *      (1) If pixs is null, this generates an empty pix of a size determined
+ *          by union of the component bounding boxes, and including the origin.
  *      (2) The selected component is blitted into pixs.
  */
 PIX *
@@ -857,18 +1086,23 @@ pixaRenderComponent(PIX     *pixs,
                     PIXA    *pixa,
                     l_int32  index)
 {
-l_int32  n, x, y, w, h;
+l_int32  n, x, y, w, h, maxdepth;
 BOX     *box;
 BOXA    *boxa;
 PIX     *pix;
 
-    PROCNAME("pixRenderComponent");
+    PROCNAME("pixaRenderComponent");
 
     if (!pixa)
         return (PIX *)ERROR_PTR("pixa not defined", procName, pixs);
     n = pixaGetCount(pixa);
     if (index < 0 || index >= n)
         return (PIX *)ERROR_PTR("invalid index", procName, pixs);
+    if (pixs && (pixGetDepth(pixs) != 1))
+        return (PIX *)ERROR_PTR("pixs not 1 bpp", procName, pixs);
+    pixaVerifyDepth(pixa, &maxdepth);
+    if (maxdepth > 1)
+        return (PIX *)ERROR_PTR("not all pix with d == 1", procName, pixs);
 
     boxa = pixaGetBoxa(pixa, L_CLONE);
     if (!pixs) {
@@ -990,7 +1224,7 @@ PIXA    *pixad;
             numaAddNumber(na, (l_float32)w / (l_float32)h);
             break;
         default:
-            L_WARNING("invalid sort type", procName);
+            L_WARNING("invalid sort type\n", procName);
         }
     }
 
@@ -1091,7 +1325,7 @@ PIXA    *pixad;
             numaAddNumber(na, w + h);
             break;
         default:
-            L_WARNING("invalid sort type", procName);
+            L_WARNING("invalid sort type\n", procName);
         }
     }
 
@@ -1159,7 +1393,7 @@ PIXA    *pixad;
  *      Input:  pixas
  *              naa (numaa that maps from the new pixaa to the input pixas)
  *              copyflag (L_CLONE or L_COPY)
- *      Return: pixaa (sorted), or null on error
+ *      Return: paa (sorted), or null on error
  */
 PIXAA *
 pixaSort2dByIndex(PIXA    *pixas,
@@ -1171,7 +1405,7 @@ BOX     *box;
 NUMA    *na;
 PIX     *pix;
 PIXA    *pixa;
-PIXAA   *pixaa;
+PIXAA   *paa;
 
     PROCNAME("pixaSort2dByIndex");
 
@@ -1187,7 +1421,7 @@ PIXAA   *pixaa;
         return (PIXAA *)ERROR_PTR("element count mismatch", procName, NULL);
 
     n = numaaGetCount(naa);
-    pixaa = pixaaCreate(n);
+    paa = pixaaCreate(n);
     for (i = 0; i < n; i++) {
         na = numaaGetNuma(naa, i, L_CLONE);
         nn = numaGetCount(na);
@@ -1199,14 +1433,254 @@ PIXAA   *pixaa;
             pixaAddPix(pixa, pix, L_INSERT);
             pixaAddBox(pixa, box, L_INSERT);
         }
-        pixaaAddPixa(pixaa, pixa, L_INSERT);
+        pixaaAddPixa(paa, pixa, L_INSERT);
         numaDestroy(&na);
     }
 
-    return pixaa;
+    return paa;
 }
 
 
+/*---------------------------------------------------------------------*
+ *                    Pixa and Pixaa range selection                   *
+ *---------------------------------------------------------------------*/
+/*!
+ *  pixaSelectRange()
+ *
+ *      Input:  pixas
+ *              first (use 0 to select from the beginning)
+ *              last (use 0 to select to the end)
+ *              copyflag (L_COPY, L_CLONE)
+ *      Return: pixad, or null on error
+ *
+ *  Notes:
+ *      (1) The copyflag specifies what we do with each pix from pixas.
+ *          Specifically, L_CLONE inserts a clone into pixad of each
+ *          selected pix from pixas.
+ */
+PIXA *
+pixaSelectRange(PIXA    *pixas,
+                l_int32  first,
+                l_int32  last,
+                l_int32  copyflag)
+{
+l_int32  n, npix, i;
+PIX     *pix;
+PIXA    *pixad;
+
+    PROCNAME("pixaSelectRange");
+
+    if (!pixas)
+        return (PIXA *)ERROR_PTR("pixas not defined", procName, NULL);
+    if (copyflag != L_COPY && copyflag != L_CLONE)
+        return (PIXA *)ERROR_PTR("invalid copyflag", procName, NULL);
+    n = pixaGetCount(pixas);
+    first = L_MAX(0, first);
+    if (last <= 0) last = n - 1;
+    if (first >= n)
+        return (PIXA *)ERROR_PTR("invalid first", procName, NULL);
+    if (first > last)
+        return (PIXA *)ERROR_PTR("first > last", procName, NULL);
+
+    npix = last - first + 1;
+    pixad = pixaCreate(npix);
+    for (i = first; i <= last; i++) {
+        pix = pixaGetPix(pixas, i, copyflag);
+        pixaAddPix(pixad, pix, L_INSERT);
+    }
+    return pixad;
+}
+
+
+/*!
+ *  pixaaSelectRange()
+ *
+ *      Input:  paas
+ *              first (use 0 to select from the beginning)
+ *              last (use 0 to select to the end)
+ *              copyflag (L_COPY, L_CLONE)
+ *      Return: paad, or null on error
+ *
+ *  Notes:
+ *      (1) The copyflag specifies what we do with each pixa from paas.
+ *          Specifically, L_CLONE inserts a clone into paad of each
+ *          selected pixa from paas.
+ */
+PIXAA *
+pixaaSelectRange(PIXAA   *paas,
+                 l_int32  first,
+                 l_int32  last,
+                 l_int32  copyflag)
+{
+l_int32  n, npixa, i;
+PIXA    *pixa;
+PIXAA   *paad;
+
+    PROCNAME("pixaaSelectRange");
+
+    if (!paas)
+        return (PIXAA *)ERROR_PTR("paas not defined", procName, NULL);
+    if (copyflag != L_COPY && copyflag != L_CLONE)
+        return (PIXAA *)ERROR_PTR("invalid copyflag", procName, NULL);
+    n = pixaaGetCount(paas, NULL);
+    first = L_MAX(0, first);
+    if (last <= 0) last = n - 1;
+    if (first >= n)
+        return (PIXAA *)ERROR_PTR("invalid first", procName, NULL);
+    if (first > last)
+        return (PIXAA *)ERROR_PTR("first > last", procName, NULL);
+
+    npixa = last - first + 1;
+    paad = pixaaCreate(npixa);
+    for (i = first; i <= last; i++) {
+        pixa = pixaaGetPixa(paas, i, copyflag);
+        pixaaAddPixa(paad, pixa, L_INSERT);
+    }
+    return paad;
+}
+
+
+/*---------------------------------------------------------------------*
+ *                        Pixa and Pixaa scaling                       *
+ *---------------------------------------------------------------------*/
+/*!
+ *  pixaaScaleToSize()
+ *
+ *      Input:  paas
+ *              wd  (target width; use 0 if using height as target)
+ *              hd  (target height; use 0 if using width as target)
+ *      Return: paad, or null on error
+ *
+ *  Notes:
+ *      (1) This guarantees that each output scaled image has the
+ *          dimension(s) you specify.
+ *           - To specify the width with isotropic scaling, set @hd = 0.
+ *           - To specify the height with isotropic scaling, set @wd = 0.
+ *           - If both @wd and @hd are specified, the image is scaled
+ *             (in general, anisotropically) to that size.
+ *           - It is an error to set both @wd and @hd to 0.
+ */
+PIXAA *
+pixaaScaleToSize(PIXAA   *paas,
+                 l_int32  wd,
+                 l_int32  hd)
+{
+l_int32  n, i;
+PIXA    *pixa1, *pixa2;
+PIXAA   *paad;
+
+    PROCNAME("pixaaScaleToSize");
+
+    if (!paas)
+        return (PIXAA *)ERROR_PTR("paas not defined", procName, NULL);
+    if (wd <= 0 && hd <= 0)
+        return (PIXAA *)ERROR_PTR("neither wd nor hd > 0", procName, NULL);
+
+    n = pixaaGetCount(paas, NULL);
+    paad = pixaaCreate(n);
+    for (i = 0; i < n; i++) {
+        pixa1 = pixaaGetPixa(paas, i, L_CLONE);
+        pixa2 = pixaScaleToSize(pixa1, wd, hd);
+        pixaaAddPixa(paad, pixa2, L_INSERT);
+        pixaDestroy(&pixa1);
+    }
+    return paad;
+}
+
+
+/*!
+ *  pixaaScaleToSizeVar()
+ *
+ *      Input:  paas
+ *              nawd  (<optional> target widths; use NULL if using height)
+ *              nahd  (<optional> target height; use NULL if using width)
+ *      Return: paad, or null on error
+ *
+ *  Notes:
+ *      (1) This guarantees that the scaled images in each pixa have the
+ *          dimension(s) you specify in the numas.
+ *           - To specify the width with isotropic scaling, set @nahd = NULL.
+ *           - To specify the height with isotropic scaling, set @nawd = NULL.
+ *           - If both @nawd and @nahd are specified, the image is scaled
+ *             (in general, anisotropically) to that size.
+ *           - It is an error to set both @nawd and @nahd to NULL.
+ *      (2) If either nawd and/or nahd is defined, it must have the same
+ *          count as the number of pixa in paas.
+ */
+PIXAA *
+pixaaScaleToSizeVar(PIXAA  *paas,
+                    NUMA   *nawd,
+                    NUMA   *nahd)
+{
+l_int32  n, i, wd, hd;
+PIXA    *pixa1, *pixa2;
+PIXAA   *paad;
+
+    PROCNAME("pixaaScaleToSizeVar");
+
+    if (!paas)
+        return (PIXAA *)ERROR_PTR("paas not defined", procName, NULL);
+    if (!nawd && !nahd)
+        return (PIXAA *)ERROR_PTR("!nawd && !nahd", procName, NULL);
+
+    n = pixaaGetCount(paas, NULL);
+    if (nawd && (n != numaGetCount(nawd)))
+        return (PIXAA *)ERROR_PTR("nawd wrong size", procName, NULL);
+    if (nahd && (n != numaGetCount(nahd)))
+        return (PIXAA *)ERROR_PTR("nahd wrong size", procName, NULL);
+    paad = pixaaCreate(n);
+    for (i = 0; i < n; i++) {
+        wd = hd = 0;
+        if (nawd) numaGetIValue(nawd, i, &wd);
+        if (nahd) numaGetIValue(nahd, i, &hd);
+        pixa1 = pixaaGetPixa(paas, i, L_CLONE);
+        pixa2 = pixaScaleToSize(pixa1, wd, hd);
+        pixaaAddPixa(paad, pixa2, L_INSERT);
+        pixaDestroy(&pixa1);
+    }
+    return paad;
+}
+
+
+/*!
+ *  pixaScaleToSize()
+ *
+ *      Input:  pixas
+ *              wd  (target width; use 0 if using height as target)
+ *              hd  (target height; use 0 if using width as target)
+ *      Return: pixad, or null on error
+ *
+ *  Notes:
+ *      (1) See pixaaScaleToSize()
+ */
+PIXA *
+pixaScaleToSize(PIXA    *pixas,
+                l_int32  wd,
+                l_int32  hd)
+{
+l_int32  n, i;
+PIX     *pix1, *pix2;
+PIXA    *pixad;
+
+    PROCNAME("pixaScaleToSize");
+
+    if (!pixas)
+        return (PIXA *)ERROR_PTR("pixas not defined", procName, NULL);
+    if (wd <= 0 && hd <= 0)
+        return (PIXA *)ERROR_PTR("neither wd nor hd > 0", procName, NULL);
+
+    n = pixaGetCount(pixas);
+    pixad = pixaCreate(n);
+    for (i = 0; i < n; i++) {
+        pix1 = pixaGetPix(pixas, i, L_CLONE);
+        pix2 = pixScaleToSize(pix1, wd, hd);
+        pixCopyText(pix2, pix1);
+        pixaAddPix(pixad, pix2, L_INSERT);
+        pixDestroy(&pix1);
+    }
+    return pixad;
+}
+
 
 /*---------------------------------------------------------------------*
  *                        Miscellaneous functions                      *
@@ -1278,7 +1752,7 @@ PIX     *pixs, *pixd;
     boxad = pixaGetBoxa(pixad, L_CLONE);
     for (i = 0; i < nbox; i++) {
         if ((box = pixaGetBox(pixas, i, L_COPY)) == NULL) {
-            L_WARNING_INT("box %d not found", procName, i);
+            L_WARNING("box %d not found\n", procName, i);
             break;
         }
         boxAdjustSides(box, box, -left, right, -top, bot);
@@ -1296,7 +1770,7 @@ PIX     *pixs, *pixd;
 /*!
  *  pixaaFlattenToPixa()
  *
- *      Input:  pixaa
+ *      Input:  paa
  *              &naindex  (<optional return> the pixa index in the pixaa)
  *              copyflag  (L_COPY or L_CLONE)
  *      Return: pixa, or null on error
@@ -1308,11 +1782,11 @@ PIX     *pixs, *pixd;
  *          each pix in the pixaa, the index of the pixa to which it belongs.
  */
 PIXA *
-pixaaFlattenToPixa(PIXAA   *pixaa,
+pixaaFlattenToPixa(PIXAA   *paa,
                    NUMA   **pnaindex,
                    l_int32  copyflag)
 {
-l_int32  i, j, m, n;
+l_int32  i, j, m, mb, n;
 BOX     *box;
 NUMA    *naindex;
 PIX     *pix;
@@ -1321,8 +1795,8 @@ PIXA    *pixa, *pixat;
     PROCNAME("pixaaFlattenToPixa");
 
     if (pnaindex) *pnaindex = NULL;
-    if (!pixaa)
-        return (PIXA *)ERROR_PTR("pixaa not defined", procName, NULL);
+    if (!paa)
+        return (PIXA *)ERROR_PTR("paa not defined", procName, NULL);
     if (copyflag != L_COPY && copyflag != L_CLONE)
         return (PIXA *)ERROR_PTR("invalid copyflag", procName, NULL);
 
@@ -1331,16 +1805,19 @@ PIXA    *pixa, *pixat;
         *pnaindex = naindex;
     }
 
-    n = pixaaGetCount(pixaa);
+    n = pixaaGetCount(paa, NULL);
     pixa = pixaCreate(n);
     for (i = 0; i < n; i++) {
-        pixat = pixaaGetPixa(pixaa, i, L_CLONE);
+        pixat = pixaaGetPixa(paa, i, L_CLONE);
         m = pixaGetCount(pixat);
+        mb = pixaGetBoxaCount(pixat);
         for (j = 0; j < m; j++) {
             pix = pixaGetPix(pixat, j, copyflag);
-            box = pixaGetBox(pixat, j, copyflag);
             pixaAddPix(pixa, pix, L_INSERT);
-            pixaAddBox(pixa, box, L_INSERT);
+            if (j < mb) {
+                box = pixaGetBox(pixat, j, copyflag);
+                pixaAddBox(pixa, box, L_INSERT);
+            }
             if (pnaindex)
                 numaAddNumber(naindex, i);  /* save 'row' number */
         }
@@ -1351,6 +1828,56 @@ PIXA    *pixa, *pixat;
 }
 
 
+/*!
+ *  pixaaSizeRange()
+ *
+ *      Input:  paa
+ *              &minw, &minh, &maxw, &maxh (<optional return> range of
+ *                                          dimensions of all boxes)
+ *      Return: 0 if OK, 1 on error
+ */
+l_int32
+pixaaSizeRange(PIXAA    *paa,
+               l_int32  *pminw,
+               l_int32  *pminh,
+               l_int32  *pmaxw,
+               l_int32  *pmaxh)
+{
+l_int32  minw, minh, maxw, maxh, minpw, minph, maxpw, maxph, i, n;
+PIXA    *pixa;
+
+    PROCNAME("pixaaSizeRange");
+
+    if (!paa)
+        return ERROR_INT("paa not defined", procName, 1);
+    if (!pminw && !pmaxw && !pminh && !pmaxh)
+        return ERROR_INT("no data can be returned", procName, 1);
+
+    minw = minh = 100000000;
+    maxw = maxh = 0;
+    n = pixaaGetCount(paa, NULL);
+    for (i = 0; i < n; i++) {
+        pixa = pixaaGetPixa(paa, i, L_CLONE);
+        pixaSizeRange(pixa, &minpw, &minph, &maxpw, &maxph);
+        if (minpw < minw)
+            minw = minpw;
+        if (minph < minh)
+            minh = minph;
+        if (maxpw > maxw)
+            maxw = maxpw;
+        if (maxph > maxh)
+            maxh = maxph;
+        pixaDestroy(&pixa);
+    }
+
+    if (pminw) *pminw = minw;
+    if (pminh) *pminh = minh;
+    if (pmaxw) *pmaxw = maxw;
+    if (pmaxh) *pmaxh = maxh;
+    return 0;
+}
+
+
 /*!
  *  pixaSizeRange()
  *
@@ -1538,6 +2065,66 @@ l_int32  maxd, same;  /* depth info */
 }
 
 
+/*!
+ *  pixaConvertToSameDepth()
+ *
+ *      Input:  pixas
+ *      Return: pixad, or null on error
+ *
+ *  Notes:
+ *      (1) If any pix has a colormap, they are all converted to rgb.
+ *          Otherwise, they are all converted to the maximum depth of
+ *          all the pix.
+ *      (2) This can be used to allow lossless rendering onto a single pix.
+ */
+PIXA *
+pixaConvertToSameDepth(PIXA  *pixas)
+{
+l_int32  i, n, same, hascmap, maxdepth;
+PIX     *pix, *pixt;
+PIXA    *pixat, *pixad;
+
+    PROCNAME("pixaConvertToSameDepth");
+
+    if (!pixas)
+        return (PIXA *)ERROR_PTR("pixas not defined", procName, NULL);
+
+        /* Remove colormaps to rgb */
+    if ((n = pixaGetCount(pixas)) == 0)
+        return (PIXA *)ERROR_PTR("no components", procName, NULL);
+    pixaAnyColormaps(pixas, &hascmap);
+    if (hascmap) {
+        pixat = pixaCreate(n);
+        for (i = 0; i < n; i++) {
+            pixt = pixaGetPix(pixas, i, L_CLONE);
+            pix = pixConvertTo32(pixt);
+            pixaAddPix(pixat, pix, L_INSERT);
+            pixDestroy(&pixt);
+        }
+    } else {
+        pixat = pixaCopy(pixas, L_CLONE);
+    }
+
+    pixaGetDepthInfo(pixat, &maxdepth, &same);
+    if (!same) {  /* at least one pix has depth > 1 */
+        pixad = pixaCreate(n);
+        for (i = 0; i < n; i++) {
+            pixt = pixaGetPix(pixat, i, L_CLONE);
+            if (maxdepth <= 8)
+                pix = pixConvertTo8(pixt, 0);
+            else
+                pix = pixConvertTo32(pixt);
+            pixaAddPix(pixad, pix, L_INSERT);
+            pixDestroy(&pixt);
+        }
+    } else {
+        pixad = pixaCopy(pixat, L_CLONE);
+    }
+    pixaDestroy(&pixat);
+    return pixad;
+}
+
+
 /*!
  *  pixaEqual()
  *
